#!/usr/bin/env python
import os
import sys
import logging
import rich.console
from halo import Halo
from Bio import SeqIO
import glob
import s_rna_tools.utils

log = logging.getLogger(__name__)
stderr = rich.console.Console(
    stderr=True,
    style="dim",
    highlight=False,
    force_terminal=s_rna_tools.utils.rich_force_colors(),
)


class CreateUnique:
    def __init__(self, folder=None, mirna=None, out_file=None, in_format=None):
        if folder is None:
            folder = s_rna_tools.utils.prompt_path(
                msg="Select the fasta file to group sequences"
            )
        self.folder = folder
        if not os.path.dirname(self.folder):
            log.error("folder  %s does not exist ", self.folder)
            stderr.print(f"[red] Folder for input files {self.folder} does not exist")
            sys.exit(1)
        if mirna:
            if not s_rna_tools.utils.file_exists(mirna):
                log.error("miRNA file  %s does not exist ", mirna)
                stderr.print(f"[red] miRNA file  {mirna} does not exist")
                sys.exit(1)
        self.mirna = mirna

        if out_file is None:
            out_file = s_rna_tools.utils.prompt_path(
                msg="Select the file to save results"
            )
        self.out_file = out_file

        if in_format is None:
            in_format = s_rna_tools.utils.prompt_selection(
                "Define the input format", ["summary", "fasta"]
            )
        self.in_format = in_format
        return

    def find_unique_in_file(self, f_name, unique_seq):
        """Find new unique sequences"""
        if self.in_format == "summary":
            with open(f_name, "r") as fh:
                lines = fh.readlines()
            for line in lines:
                seq = line.split("\t")[1]
                if seq not in unique_seq:
                    unique_seq[seq] = line.split("\t")[0]
        elif self.in_format == "fasta":
            for seq_record in SeqIO.parse(f_name, "fasta"):
                if str(seq_record.seq) not in unique_seq:
                    unique_seq[str(seq_record.seq)] = str(seq_record.id)
        return unique_seq

    def read_mirna_file(self):
        """Read the miRNA file, collecting the unique sequences and convert RNA to DNA sequences"""
        mirna_unique = {}
        for seq_record in SeqIO.parse(self.mirna, "fasta"):
            seq = str(seq_record.seq.back_transcribe())
            if seq not in mirna_unique:
                mirna_unique[seq] = str(seq_record.id)
        return mirna_unique

    def remove_mirna_seq(self, sequences, mirna):
        """Remove in the unique sequences the miRNAs. miRNAs harpins are also
        discarded"""
        return

    def collect_unique(self):
        """Collect the unique sequences"""
        if self.in_format == "summary":
            file_list = glob.glob(self.folder + "*.tsv")
            if len(file_list) == 0:
                log.error("There is not valid files on folder  %s ", self.folder)
                stderr.print(f"[red] There is not valid files on folder {self.folder}")
                sys.exit(1)
        else:
            file_list = glob.glob(self.folder + "*.fa")
            if len(file_list) == 0:
                file_list = glob(self.folder, "/*.fasta")
                if len(file_list) == 0:
                    log.error("There is not valid files on folder  %s ", self.folder)
                    stderr.print(
                        f"[red] There is not valid files on folder {self.folder}"
                    )
                    sys.exit(1)

        unique_seq = {}
        stderr.print("Starting collecting unique sequences")
        spinner = Halo(spinner="dots")
        spinner.start()
        for f_name in file_list:
            unique_seq = self.find_unique_in_file(f_name, unique_seq)
        spinner.stop()
        if self.mirna:
            pass
        s_rna_tools.utils.write_unique_seq(unique_seq, self.out_file)
        stderr.print("Completed")



#!/usr/bin/env python
import os
import sys
import logging
import rich.console
from halo import Halo
from Bio import SeqIO
import s_rna_tools.utils

log = logging.getLogger(__name__)
stderr = rich.console.Console(
    stderr=True,
    style="dim",
    highlight=False,
    force_terminal=s_rna_tools.utils.rich_force_colors(),
)


class GroupSequences:
    def __init__(self, seq_file=None, out_folder=None, out_format=None, threshold=None):
        if seq_file is None:
            seq_file = s_rna_tools.utils.prompt_path(
                msg="Select the fasta file to group sequences"
            )
        self.seq_file = seq_file
        if not os.path.isfile(self.seq_file):
            log.error("fasta file %s does not exist ", self.seq_file)
            stderr.print(f"[red] fasta data file {self.seq_file} does not exist")
            sys.exit(1)
        if out_folder is None:
            out_folder = s_rna_tools.utils.prompt_path(
                msg="Select the folder to save results"
            )
        self.out_folder = out_folder

        if out_format is None:
            out_format = s_rna_tools.utils.prompt_selection(
                "Define the output format", ["summary", "fasta"]
            )
        self.out_format = out_format
        if threshold is None:
            threshold = 10
        try:
            self.threshold = int(threshold)
        except ValueError:
            log.error(
                "%s is not a valid value for threshold. Integer value must be set",
                threshold,
            )
            stderr.print(
                f"[red] {threshold} is not a valid value for threshold. Integer value must be set"
            )
            sys.exit(1)

        return

    def filter_seq(self, seqs):
        f_seqs = {}
        for seq, counter in seqs.items():
            if counter >= self.threshold:
                f_seqs[seq] = counter
        return f_seqs

    def counter_seq(self):
        seq_counter = {}
        spinner = Halo(text="Searching for repeated contings", spinner="dots")
        spinner.start()
        for seq_record in SeqIO.parse(self.seq_file, "fasta"):
            str_seq = str(seq_record.seq)
            if str_seq not in seq_counter:
                seq_counter[str_seq] = 0
            seq_counter[str_seq] += 1
        spinner.succeed("Processed sequence file")

        if self.threshold != 0:
            spinner.start("filtering sequences for removing low values")
            seq_counter = self.filter_seq(seq_counter)
            spinner.succeed("Filter done")
        heading = "SRNA\tSequence\tCounts"
        seq_id = os.path.basename(self.seq_file).split(".")[0] + "_RNA_"
        count_f_name = os.path.join(
            self.out_folder,
            os.path.basename(self.seq_file).split(".")[0] + "_count_reads",
        )
        if self.out_format == "summary":
            count_f_name += ".tsv"
        else:
            count_f_name += ".fa"
        stderr.print("[green] Saving data to file")
        s_rna_tools.utils.write_seq_file(
            seq_counter, count_f_name, heading, seq_id, self.out_format
        )
        spinner.stop()
        stderr.print(
            f"[green] Processing file {os.path.basename(self.seq_file)} completed"
        )
        return



#!/usr/bin/env python
"""
Common utility function for relecov_tools package.
"""
import os
import logging
from rich.console import Console
import questionary


log = logging.getLogger(__name__)


def file_exists(file_to_check):
    """
    Input:
        file_to_check   # file name to check if exists
    Return:
        True if exists
    """
    if os.path.isfile(file_to_check):
        return True
    return False


def write_seq_file(data, out_file, heading, seq_name, format):
    """write the sequence data dict to text file"""
    counter = 1
    with open(out_file, "w") as fh:
        if format == "summary":
            fh.write(heading + "\n")
            for key, value in data.items():
                fh.write(
                    seq_name + str(counter) + "\t" + key + "\t" + str(value) + "\n"
                )
                counter += 1
        elif format == "fasta":
            for key, value in data.items():
                fh.write("> " + seq_name + str(counter) + "_x" + str(value) + "\n")
                fh.write(key + "\n")
                counter += 1
        else:
            pass
    return


def write_unique_seq(data, out_file):
    """write the unique sequences to fasta file"""
    with open(out_file, "w") as fh:
        for key, value in data.items():
            fh.write("> " + value + "\n")
            fh.write(key + "\n")
    return


def rich_force_colors():
    """
    Check if any environment variables are set to force Rich to use coloured output
    """
    if (
        os.getenv("GITHUB_ACTIONS")
        or os.getenv("FORCE_COLOR")
        or os.getenv("PY_COLORS")
    ):
        return True
    return None


stderr = Console(
    stderr=True, style="dim", highlight=False, force_terminal=rich_force_colors()
)


def prompt_tmp_dir_path():
    stderr.print("Temporal directory destination to execute service")
    source = questionary.path("Source path").unsafe_ask()
    return source


def prompt_selection(msg, choices):
    selection = questionary.select(msg, choices=choices).unsafe_ask()
    return selection


def prompt_path(msg):
    source = questionary.path(msg).unsafe_ask()
    return source


def prompt_message(msg):
    value = questionary.text(msg).unsafe_ask()
    return value



#!/usr/bin/env python
import logging
import emoji

# from rich.prompt import Confirm
import click
import rich.console
import rich.logging
import rich.traceback

import s_rna_tools.utils
import s_rna_tools.group_sequences
import s_rna_tools.create_unique
import s_rna_tools.find_match
import s_rna_tools.countifier
import s_rna_tools.join_duplicates
import s_rna_tools.match_all_smrna

log = logging.getLogger()


def run_s_rna_tools():
    # Set up rich stderr console
    stderr = rich.console.Console(
        stderr=True, force_terminal=s_rna_tools.utils.rich_force_colors()
    )

    # Set up the rich traceback
    rich.traceback.install(console=stderr, width=200, word_wrap=True, extra_lines=1)
    stderr.print(
        "[yellow]    ____            ___                            ",
        highlight=False,
    )
    stderr.print(
        "[yellow]   /      [blue]__  __ [yellow]  |   \ |\   |    /\ ", highlight=False
    )
    stderr.print(
        "[yellow]   \___   [red]  \/   [yellow]  |__ / | \  |   /  \ ", highlight=False
    )
    stderr.print(
        "[yellow]       \  [blue]__/\__ [yellow]  |  \  |  \ |  /____\ ",
        highlight=False,
    )
    stderr.print(
        "[yellow]   ____/         [yellow]  |   \ |   \| /      \ ", highlight=False
    )

    __version__ = "0.0.1"
    stderr.print()
    stderr.print(
        "       ",
        emoji.emojize(":dna:"),
        "[bold grey39] s-rna-tools version {}".format(__version__),
        highlight=False,
    )
    stderr.print()

    # Lanch the click cli
    s_rna_tools_cli()


# Customise the order of subcommands for --help
class CustomHelpOrder(click.Group):
    def __init__(self, *args, **kwargs):
        self.help_priorities = {}
        super(CustomHelpOrder, self).__init__(*args, **kwargs)

    def get_help(self, ctx):
        self.list_commands = self.list_commands_for_help
        return super(CustomHelpOrder, self).get_help(ctx)

    def list_commands_for_help(self, ctx):
        """reorder the list of commands when listing the help"""
        commands = super(CustomHelpOrder, self).list_commands(ctx)
        return (
            c[1]
            for c in sorted(
                (self.help_priorities.get(command, 1000), command)
                for command in commands
            )
        )

    def command(self, *args, **kwargs):
        """Behaves the same as `click.Group.command()` except capture
        a priority for listing command names in help.
        """
        help_priority = kwargs.pop("help_priority", 1000)
        help_priorities = self.help_priorities

        def decorator(f):
            cmd = super(CustomHelpOrder, self).command(*args, **kwargs)(f)
            help_priorities[cmd.name] = help_priority
            return cmd

        return decorator


@click.group(cls=CustomHelpOrder)
@click.version_option(s_rna_tools.__version__)
@click.option(
    "-v",
    "--verbose",
    is_flag=True,
    default=False,
    help="Print verbose output to the console.",
)
@click.option(
    "-l", "--log-file", help="Save a verbose log to a file.", metavar="<filename>"
)
def s_rna_tools_cli(verbose, log_file):

    # Set the base logger to output DEBUG
    log.setLevel(logging.DEBUG)

    # Set up logs to a file if we asked for one
    if log_file:
        log_fh = logging.FileHandler(log_file, encoding="utf-8")
        log_fh.setLevel(logging.DEBUG)
        log_fh.setFormatter(
            logging.Formatter(
                "[%(asctime)s] %(name)-20s [%(levelname)-7s]  %(message)s"
            )
        )
        log.addHandler(log_fh)


@s_rna_tools_cli.command(help_priority=1)
@click.option("-i", "--infile", help="fasta input file with sequences")
@click.option("-o", "--out_folder", help="Path to save generated ouput files")
@click.option(
    "-f",
    "--out_format",
    type=click.Choice(["summary", "fasta"]),
    help="Select format for output file",
)
@click.option(
    "-t", "--threshold", help="Ignore the sequences founded below the threshold"
)
def group_sequences(infile, out_folder, out_format, threshold):
    """Group small RNA sequences."""
    new_s_group = s_rna_tools.group_sequences.GroupSequences(
        infile, out_folder, out_format, threshold
    )
    new_s_group.counter_seq()


@s_rna_tools_cli.command(help_priority=2)
@click.option(
    "-d", "--directory", type=click.Path(), help="Folder with processed count files"
)
@click.option(
    "-m",
    "--mirna",
    type=click.Path(),
    help="miRNA fasta file to exclude them in unique sequences",
)
@click.option(
    "-o", "--out_file", help="file name to save fata file for unique sequences"
)
@click.option(
    "-f",
    "--out_format",
    type=click.Choice(["summary", "fasta"]),
    help="Select format for output file",
)
def create_unique(directory, mirna, out_file, out_format):
    """Create a file which has only the unique samples"""
    unique_seq = s_rna_tools.create_unique.CreateUnique(
        directory, mirna, out_file, out_format
    )
    unique_seq.collect_unique()


@s_rna_tools_cli.command(help_priority=3)
@click.option("-i", "--in_seq", type=click.Path(), help="file having the unique reads")
@click.option(
    "-m",
    "--match_to_file",
    type=click.Path(),
    help="file with the sequences to discard",
)
@click.option(
    "-o", "--outdir", type=click.Path(), help="output directory to save result"
)
@click.option(
    "-l",
    "--known_list",
    is_flag=True,
    default=None,
    help="Create file with the found matched sequence ids",
)
@click.option(
    "-k",
    "--known_match",
    type=click.Choice(["known", "unknown"]),
    help="Select format for output file",
)
def find_match(in_seq, match_to_file, outdir, known_list, known_match):
    """Find sequences that match with the selected files"""
    new_findings = s_rna_tools.find_match.FindMatch(
        in_seq, match_to_file, outdir, known_list, known_match
    )
    new_findings.get_match_sequences()


@s_rna_tools_cli.command(help_priority=4)
@click.option(
    "-u", "--unique_seq", type=click.Path(), help="file having the unique reads"
)
@click.option(
    "-s",
    "--sample_file",
    type=click.Path(),
    help="file with the count reads sequences",
)
@click.option(
    "-o", "--outdir", type=click.Path(), help="output directory to save result"
)
def countifier(sample_file, unique_seq, outdir):
    new_cuantifier = s_rna_tools.countifier.Countifier(sample_file, unique_seq, outdir)
    new_cuantifier.split_files()


@s_rna_tools_cli.command(help_priority=5)
@click.option(
    "-i",
    "--input",
    type=click.Path(),
    help="file having duplicated value in a key value",
)
@click.option(
    "-p", "position", type=click.INT, help="position where checking if duplicated"
)
def join_duplicates(input, position):
    unique_count = s_rna_tools.join_duplicates.JoinDuplicates(input, position)
    result = unique_count.make_unique()
    unique_count.write_join(result)


@s_rna_tools_cli.command(help_priority=6)
@click.option(
    "-r", "--grep_result", type=click.Path(), help="file having the grep result"
)
@click.option(
    "-s", "--sample_file", type=click.Path(), help="file having the sequences counts per sample"
)
@click.option(
    "-o", "--outdir", type=click.Path(), help="output directory to save result"
)
def match_all_smrna(grep_result, sample_file, outdir):
    match_smrna = s_rna_tools.match_all_smrna.MatchAllSmrna(grep_result, sample_file, outdir)
    match_smrna.get_match_sequences()



#!/usr/bin/env python
import os
import sys
import logging
import rich.console
from Bio import SeqIO
import s_rna_tools.utils
from s_rna_tools.rna_blast import RnaBlast
from halo import Halo

log = logging.getLogger(__name__)
stderr = rich.console.Console(
    stderr=True,
    style="dim",
    highlight=False,
    force_terminal=s_rna_tools.utils.rich_force_colors(),
)


class FindMatch:
    def __init__(
        self,
        in_file=None,
        m_file=None,
        out_folder=None,
        known_list=None,
        known_match="known",
    ):
        if in_file is None:
            in_file = s_rna_tools.utils.prompt_path(
                msg="Select the fasta file with the sequences"
            )
        self.in_file = in_file
        if not os.path.isfile(self.in_file):
            log.error("fasta file %s does not exist ", self.in_file)
            stderr.print(f"[red] fasta data file {self.in_file} does not exist")
            sys.exit(1)
        if m_file is None:
            m_file = s_rna_tools.utils.prompt_path(
                msg="Select the file to match against the sequences"
            )
        self.m_file = m_file
        if not os.path.isfile(self.m_file):
            log.error("matching file %s does not exist ", self.m_file)
            stderr.print(f"[red] matching file {self.m_file} does not exist")
            sys.exit(1)
        if out_folder is None:
            out_folder = s_rna_tools.utils.prompt_path(
                msg="Select the folder to save results"
            )
        self.out_folder = out_folder
        self.known_match = known_match
        self.create_known_list = known_list

    def create_blast_instance(self):
        return RnaBlast(self.in_file, "blastn-short", 100, 0.05, self.out_folder)

    def extract_sequences(self):
        sequences = {}
        for seq_record in SeqIO.parse(self.in_file, "fasta"):
            sequences[str(seq_record.seq)] = str(seq_record.id)
        return sequences

    def get_not_matched(self, sequences, f_name):
        """Read the file which contains the found matches and discard them from
        the unique m_sequences
        """
        with open(f_name, "r") as fh:
            lines = fh.readlines()
        seq_in_file = []
        for line in lines[1:]:
            seq_in_file.append(line.split("\t")[2])
        not_match_id = list(
            set(sequences.values()).symmetric_difference(set(seq_in_file))
        )
        return dict.fromkeys(not_match_id)

    def get_known_items(self, data):
        """From blast ouput get the known id and create a list without duplication"""
        k_found = []
        for row in data:
            k_found.append(row[0])
        return list(set(k_found))

    def write_know_items_to_file(self, data, f_name):
        """Write the known items to file"""
        with open(f_name, "w") as fh:
            for item in data:
                fh.write(item + "\n")
        return

    def write_unknown_to_file(self, data, f_name):
        """Write the unknown sequences in fasta format"""
        with open(f_name, "w") as fh:
            for id, seq in data.items():
                fh.write("> " + id + "\n")
                fh.write(seq + "\n")
        return

    def get_match_sequences(self):
        spinner = Halo(text="Creating Blast index", spinner="dots")
        spinner.start()
        blast_obj = self.create_blast_instance()
        spinner.succeed("Created index")
        spinner.start("Executing blast")
        in_sequences = self.extract_sequences()
        blast_res = blast_obj.run_blast(self.m_file)
        spinner.succeed("Blast completed")
        spinner.stop()
        if "match" in blast_res:
            sample = os.path.basename(self.in_file).split(".")[0]
            # Rename the dictonary key value
            blast_res[sample] = blast_res.pop("match")
            out_file = os.path.join(self.out_folder, sample + "blast_results.tsv")
            blast_obj.write_to_file(blast_res, out_file)
            if self.create_known_list:
                k_list = self.get_known_items(blast_res[sample])
                k_file = os.path.join(self.out_folder, sample + "know_items.txt")
                stderr.print("[green] Writting know items to file")
                self.write_know_items_to_file(k_list, k_file)
        else:
            stderr.print("[red] Not found any match with the query file")

        if self.known_match == "unknown":
            not_matched = self.get_not_matched(in_sequences, out_file)
            u_sequences_dict = {}
            for seq, id in in_sequences.items():
                if id in not_matched:
                    u_sequences_dict[id] = seq
            f_name = os.path.join(self.out_folder, "unknown_sequences.fa")
            self.write_unknown_to_file(u_sequences_dict, f_name)
        stderr.print("[green] Completed")
        return



#!/usr/bin/env python
from Bio.Blast.Applications import NcbiblastnCommandline
import os
import sys
import subprocess
import logging
import rich.console
import s_rna_tools.utils


log = logging.getLogger(__name__)
stderr = rich.console.Console(
    stderr=True,
    style="dim",
    highlight=False,
    force_terminal=s_rna_tools.utils.rich_force_colors(),
)


class RnaBlast:
    def __init__(self, blast_fasta, task, perc_identity, evalue, out_dir):
        self.blast_db = blast_fasta
        self.task = task
        self.perc_identity = perc_identity
        self.evalue = evalue
        db_name = os.path.basename(self.blast_db).split(".")[0]
        self.db_folder = os.path.join(out_dir, db_name, db_name)
        blastn_db = (
            "makeblastdb -in "
            + blast_fasta
            + " -dbtype nucl -input_type fasta -out "
            + self.db_folder
        )
        # stderr.print("Starting blast index generation. Please wait")

        proc = subprocess.Popen(
            blastn_db,
            shell=True,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )

        proc.wait()
        if proc.returncode == 0:
            stderr.print("\nSuccessful creation of blast database")
        else:
            log.error("Unable to create blast database. Error %s ", proc.returncode)
            log.error("running the following parameters %s", proc.args)
            stderr.print(f"\n[red] Unable to create blast database {proc.returncode}")
            stderr.print(f"[red] running the following parameters {proc.args}")
            sys.exit(1)

    def collect_data(self, out_lines):
        matches_found = []
        for out_line in out_lines:
            matches_found.append(out_line.split("\t"))
        return matches_found

    def run_blast(self, query_file):
        blast_parameters = '"6 , qseqid , sseqid , pident ,  qlen , length , mismatch , gapopen , evalue , bitscore , sstart , send , qstart , qend , sseq , qseq"'
        result = {}
        cline = NcbiblastnCommandline(
            db=self.db_folder,
            task=self.task,
            evalue=self.evalue,
            penalty=-2,
            perc_identity=self.perc_identity,
            outfmt=blast_parameters,
            max_target_seqs=200,
            max_hsps=20,
            num_threads=8,
            query=query_file,
        )
        out, err = cline()
        out_lines = out.splitlines()
        if len(out_lines) > 0:
            result["match"] = self.collect_data(out_lines)
        else:
            result["not_match"] = query_file

        return result

    def write_to_file(self, data, file_name):
        match_heading = "Sample\tmiRNA Name\tcontig\tpident\tmiRNA\tlength\tmismatch\tgapopen\tevalue\tbitscore\tSample start\tSample end\tmiRNA start\tmiRNA end\tSample Sequence\tmiRNA Sequence"
        with open(file_name, "w") as fh:
            fh.write(match_heading + "\n")
            for key, values in data.items():
                for value in values:
                    fh.write(key + "\t" + "\t".join(value) + "\n")
        return



#!/usr/bin/env python
import os
import sys
import logging
import rich.console
from Bio import SeqIO
import s_rna_tools.utils
from s_rna_tools.rna_blast import RnaBlast
from halo import Halo

log = logging.getLogger(__name__)
stderr = rich.console.Console(
    stderr=True,
    style="dim",
    highlight=False,
    force_terminal=s_rna_tools.utils.rich_force_colors(),
)


class MatchAllSmrna:
    def __init__(
        self,
        grep_result=None,
        sample_file=None,
        out_folder=None
    ):
        if grep_result is None:
            grep_result = s_rna_tools.utils.prompt_path(
                msg="Select the file with the grep result"
            )
        self.grep_result = grep_result
        if not os.path.isfile(self.grep_result):
            log.error("grep result file %s does not exist ", self.grep_result)
            stderr.print(f"[red] grep result file {self.grep_result} does not exist")
            sys.exit(1)

        if sample_file is None:
            sample_file = s_rna_tools.utils.prompt_path(
                msg="Select the file to match against the sequences"
            )
        self.sample_file = sample_file
        if not os.path.isfile(self.sample_file):
            log.error("matching file %s does not exist ", self.sample_file)
            stderr.print(f"[red] matching file {self.sample_file} does not exist")
            sys.exit(1)
        if out_folder is None:
            out_folder = s_rna_tools.utils.prompt_path(
                msg="Select the folder to save results"
            )
        self.out_folder = out_folder

    def _split_matched_annotation(self):
        """Read the file from the grep result and split between matched and
        not matched
        """
        with open (self.grep_result, "r") as fh:
            lines = fh.readlines()
        smrna_annot = {"match": {}, "not_match": []}
        for line in lines:
            line = line.strip()
            line_split = line.split(",")
            if "Not match" in line_split[1]:
                smrna_annot["not_match"].append(line_split[0])
            else:
                smrna_annot["match"][line_split[0]] = line_split[2]
        return smrna_annot

    def _replace_match(self, s_annot):
        """Read the sample file and replace sequence by annotations"""
        with open(self.sample_file, "r") as fh:
            lines = fh.readlines()
        mapped_list =  []
        unmapped_list = []
        mapped_list.append(lines[0])
        unmapped_list.append(lines[0])
        for line in lines[1:]:
            line_split = line.split("\t")
            if line_split[0] in s_annot:
                line_split[0] = s_annot[line_split[0]]
                mapped_list.append(",".join(line_split))
            else:
                unmapped_list.append(line)

        return mapped_list, unmapped_list

    def _write_files(self, mapped, unmapped):
        map_file = os.path.join(self.out_folder, "mapped_results.csv")
        unmap_file = os.path.join(self.out_folder, "unmapped_results.csv")
        with open (map_file, "w") as fh:
            for line in mapped:
                fh.write(line)

        with open(unmap_file, "w") as fh:
            for line in unmapped:
                fh.write(line)
        return

    def get_match_sequences(self):
        smrna_annot = self._split_matched_annotation()
        mapped, unmapped = self._replace_match(smrna_annot["match"])
        self._write_files(mapped, unmapped)


        import pdb; pdb.set_trace()



#!/usr/bin/env python
import os
import sys
import logging
import rich.console
import s_rna_tools.utils
from collections import OrderedDict

log = logging.getLogger(__name__)
stderr = rich.console.Console(
    stderr=True,
    style="dim",
    highlight=False,
    force_terminal=s_rna_tools.utils.rich_force_colors(),
)


class JoinDuplicates:
    def __init__(self, input=None, position=None):
        if input is None:
            input = s_rna_tools.utils.prompt_path(
                msg="Select the file which have a key colunm with duplicated values"
            )
        if not s_rna_tools.utils.file_exists(input):
            log.error("file  %s does not exist ", self.input)
            stderr.print(f"[red] file {self.folder} does not exist")
            sys.exit(1)
        self.input = input
        if position is None:
            position = s_rna_tools.utils.prompt_message(
                msg="Select the position where checking duplicates"
            )
        try:
            self.position = int(position)
        except ValueError:
            log.error("Position value  %s is not a integer ", position)
            stderr.print(f"[red] Position value {position} is not a integer")
            sys.exit(1)

    def join_lines(self, old_line, new_split_line):
        s_old_line = old_line.split()
        j_values = []
        for idx in range(len(s_old_line)):
            if idx <= self.position:
                j_values.append(s_old_line[idx])
                continue
            try:
                sum_values = int(s_old_line[idx]) + int(new_split_line[idx])
                j_values.append(str(sum_values))
            except ValueError:
                j_values.append(s_old_line)
        return "\t".join(j_values)

    def write_join(self, data_dict):
        f_name = os.path.join(os.path.dirname(self.input), "without_duplicated.tsv")
        with open(f_name, "w") as fh:
            for key, value in data_dict.items():
                fh.write(value + "\n")
        return

    def make_unique(self):
        with open(self.input, "r") as fh:
            lines = fh.readlines()
        s_rna_dict = OrderedDict()
        for line in lines:
            line = line.strip()
            l_split = line.split("\t")
            c_value = l_split[self.position]
            if c_value not in s_rna_dict:
                s_rna_dict[c_value] = line
            else:
                s_rna_dict[c_value] = self.join_lines(s_rna_dict[c_value], l_split)
        return s_rna_dict



#!/usr/bin/env python
import os
import sys
import re
import logging
import rich.console
from glob import glob
from Bio import SeqIO
import s_rna_tools.utils

log = logging.getLogger(__name__)
stderr = rich.console.Console(
    stderr=True,
    style="dim",
    highlight=False,
    force_terminal=s_rna_tools.utils.rich_force_colors(),
)


class Countifier:
    def __init__(self, sample_files=None, unique_seq=None, out_folder=None):
        if sample_files is None:
            sample_files = s_rna_tools.utils.prompt_path(
                msg="Select the sample file or directory with the count reads"
            )
        if os.path.isdir(sample_files):
            file_list = glob(sample_files + "/*.fa")
            if len(file_list) == 0:
                file_list = glob(sample_files + "/*.fasta")
                if len(file_list) == 0:
                    log.error(
                        "folder  %s does not contain any fasta file", self.sample_files
                    )
                    stderr.print(
                        f"[red] Folder  {self.sample_files} does not contain any fasta file"
                    )
                    sys.exit(1)
            self.file_list = file_list
        elif os.path.isfile(sample_files):
            self.file_list = [sample_files]
        else:
            log.error(" %s does not exist", self.sample_files)
            stderr.print(f"[red] {self.sample_files} does not exist")
            sys.exit(1)
        if unique_seq is None:
            unique_seq = s_rna_tools.utils.prompt_path(
                msg="Select the file which contains the unique sequences"
            )
        if not os.path.isfile(unique_seq):
            log.error(" %s does not exist", self.unique_seq)
            stderr.print(f"[red] {self.unique_seq} does not exist")
            sys.exit(1)
        self.unique_seq = unique_seq
        if out_folder is None:
            out_folder = s_rna_tools.utils.prompt_path(
                msg="Select the directory to save results"
            )
        self.out_folder = out_folder

    def parse_seq(self, fasta_file):
        """Read the unique sequence file and return a dictonary with the sequences"""
        seq_dict = {}
        for seq_record in SeqIO.parse(fasta_file, "fasta"):
            id_match = re.search(r".*_x(\d+)$", str(seq_record.id))
            if id_match:
                seq_dict[str(seq_record.seq)] = id_match.group(1)
            else:
                seq_dict[str(seq_record.seq)] = 0
        return seq_dict

    def fetch_sample_counts(self, s_file, seq_master):
        """Fetch the sequence and the count for sample, using as unique_seq
        as reference
        """
        seq_sample = self.parse_seq(s_file)
        count_seq = {}
        for seq in seq_master.keys():
            if seq in seq_sample:
                count_seq[seq] = seq_sample[seq]
            else:
                count_seq[seq] = "0"
        return count_seq

    def create_counter_file(self, samples_dict, seq_master):
        """Create the tab file with sequences  and the counter"""
        sample_list = list(samples_dict.keys())
        heading = "Sequence\tTotal Counts\t" + "\t".join(sample_list)
        f_name = os.path.join(self.out_folder, "sample_cuantifier.tsv")
        with open(f_name, "w") as fh:
            fh.write(heading + "\n")
            for seq in seq_master.keys():
                count = 0
                value_list = []
                for sample in sample_list:
                    value = samples_dict[sample][seq]
                    value_list.append(value)
                    count += int(value)
                fh.write(seq + "\t" + str(count) + "\t" + "\t".join(value_list) + "\n")
        return

    def split_files(self):
        """Read the unique sequence file, find the sequence in the sample count
        reads and create a file having the number that the sequence was found
        """
        seq_master = self.parse_seq(self.unique_seq)
        samples_dict = {}
        for s_file in self.file_list:
            f_name = os.path.basename(s_file).split(".")[0]
            f_match = re.search(r"(.*)_count_reads.*", f_name)
            if f_match:
                s_name = f_match.group(1)
            else:
                s_name = f_name
            samples_dict[s_name] = self.fetch_sample_counts(s_file, seq_master)
        self.create_counter_file(samples_dict, seq_master)
